<?php
namespace Zeedhi\Framework\DBAL;

use Doctrine\DBAL\Query\QueryBuilder as DoctrineQueryBuilder;

class QueryBuilder extends DoctrineQueryBuilder {
    /* @var array The query comments. */
    private $_comment = [];
    /* @var boolean Defines if the SQL will build the stack trace. */
    private $_debugDB = false;

    public function __construct(Connection $connection, $debugDB = false){
        $this->_debugDB = $debugDB;
        parent::__construct($connection);
    }

    /**
     * Builds the stackTrace as an array of strings.
     *
     * @return array $stackTrace the humanreadble stack trace.
     */
    private function getStackTrace(){
        $stackTrace = [];
        foreach (debug_backtrace() as $key => $traceElement) {
            if(isset($traceElement["class"]))
                $stackTrace[] = $traceElement["class"];
            if(isset($traceElement["type"]))
                $stackTrace[] = $traceElement["type"];
            if(isset($traceElement["function"]))
                $stackTrace[] = $traceElement["function"]." ";
            if(isset($traceElement["file"]))
                $stackTrace[] = "called by ".$traceElement["file"]." ";
            if(isset($traceElement["line"]))
                $stackTrace[] = "at line ".$traceElement["line"]." ";
        }
        return $stackTrace;
    }

    /**
     * {@inheritdoc}
     */
    public function execute(){
        if($this->_debugDB){
            $this->comment($this->getStackTrace());
        }
        return parent::execute();
    }

    /**
     * {@inheritdoc}
     */
    public function getSQL(){
        $sql = "";

        if(count($this->_comment)){
            $sql.= " /* ";
            foreach($this->_comment as $comment){
                $sql.= $comment;
            }
            $sql.= " */ ";
        }

        return $sql.parent::getSQL();
    }

    /**
     * {@inheritdoc}
     */
    public function add($sqlPartName, $sqlPart, $append = false){
        if($sqlPartName == 'comment'){
            foreach ($sqlPart as $comment) {
                $this->_comment[] = $comment;
            }
            return $this;
        } else {
            return parent::add($sqlPartName, $sqlPart, $append);
        }
    }

    /**
     * Adds a comment to the query.
     *
     * @param array The array of comments to add.
     *
     * @return QueryBuilder Return the QueryBuilder object.
     */
    public function comment($comment){
        return $this->add('comment', $comment);
    }
}